# password-prompt

Node cross-platform password prompt.

Supported Environments:

* Unix
* TTY or non-TTY
* cygwin
* Git Bash
* cmd.exe
* PowerShell
* cmder
* Console2

# API Reference
{{#module name="password-prompt"~}}
{{>body~}}
{{>member-index~}}
{{>members~}}
{{/module}}
